export function isActualTableCell(cell) {
    return cell !== '' && Boolean(Object.keys(cell).length);
}
/**
 * Returns a set of `rowIndex-columnIndex` keys for cells that are covered
 * by the `colSpan` and/or `rowSpan` of another cell.
 *
 * pdfmake forces us to put "empty" cells there, but then ignores them.
 */
export function getTableCellsCoveredBySpans(node) {
    var _a, _b;
    const coveredCells = new Set();
    (_b = (_a = node.table) === null || _a === void 0 ? void 0 : _a.body) === null || _b === void 0 ? void 0 : _b.forEach((row, rowIndex) => {
        row.forEach((column, columnIndex) => {
            var _a, _b;
            if (isActualTableCell(column)) {
                if ((column.rowSpan && column.rowSpan > 1) ||
                    (column.colSpan && column.colSpan > 1)) {
                    for (let i = 0; i < ((_a = column.rowSpan) !== null && _a !== void 0 ? _a : 1); i++) {
                        for (let j = 0; j < ((_b = column.colSpan) !== null && _b !== void 0 ? _b : 1); j++) {
                            if (i !== 0 || j !== 0)
                                coveredCells.add(`${rowIndex + i}-${columnIndex + j}`);
                        }
                    }
                }
            }
        });
    });
    return coveredCells;
}
export function shouldRenderCell(cell, rowIndex, columnIndex, coveredCells) {
    return !coveredCells.has(`${rowIndex}-${columnIndex}`);
}
