import { opacityColor } from './utils';
export function getCanvasProperties(node) {
    let width = 1;
    let height = 1;
    if (Array.isArray(node.canvas)) {
        node.canvas.forEach(element => {
            var _a;
            if ('w' in element) {
                const w = element.x + element.w;
                if (w > width)
                    width = w;
                const h = element.y + element.h;
                if (h > height)
                    height = h;
            }
            else if ('r1' in element) {
                const w = element.x + element.r1;
                if (w > width)
                    width = w;
                const h = element.y + ((_a = element.r2) !== null && _a !== void 0 ? _a : element.r1);
                if (h > height)
                    height = h;
            }
            else if ('x1' in element) {
                const w = Math.max(element.x1, element.x2);
                if (w > width)
                    width = w;
                const h = Math.max(element.y1, element.y2);
                if (h > height)
                    height = h;
            }
            else if ('points' in element && Array.isArray(element.points)) {
                element.points.forEach(point => {
                    if (!point)
                        return;
                    if (point.x > width)
                        width = point.x;
                    if (point.y > height)
                        height = point.y;
                });
            }
        });
    }
    return {
        width: width + 'pt',
        height: height + 'pt',
    };
}
function getCanvasFilledElementProperties(element, index) {
    const properties = {
        fill: 'none',
    };
    if (element.linearGradient) {
        properties.fill = `url('#gradient${index}')`;
    }
    else if (element.color) {
        const color = opacityColor(element.color, element.fillOpacity);
        if (color)
            properties.fill = color;
    }
    return properties;
}
function getCanvasLineElementProperties(element) {
    var _a, _b;
    if (!element.lineColor &&
        ('color' in element || 'linearGradient' in element)) {
        return {};
    }
    const properties = {
        stroke: (_a = element.lineColor) !== null && _a !== void 0 ? _a : '#000',
        'stroke-width': ((_b = element.lineWidth) !== null && _b !== void 0 ? _b : 1) + 'pt',
    };
    if (element.strokeOpacity !== undefined) {
        properties['stroke-opacity'] = String(element.strokeOpacity);
    }
    if (element.lineJoin) {
        properties['stroke-linejoin'] = element.lineJoin;
    }
    if (element.dash && typeof element.dash === 'object') {
        const { space, length } = element.dash;
        properties['stroke-dasharray'] = `${length}pt,${space !== null && space !== void 0 ? space : length}pt`;
    }
    return properties;
}
export function getCanvasRectProperties(rect, index) {
    const properties = Object.assign(Object.assign(Object.assign({}, getCanvasFilledElementProperties(rect, index)), getCanvasLineElementProperties(rect)), { x: rect.x + 'pt', y: rect.y + 'pt', width: Math.max(rect.w, rect.h ? 1 : 0) + 'pt', height: Math.max(rect.h, rect.w ? 1 : 0) + 'pt' });
    if (rect.r) {
        properties.rx = rect.r + 'pt';
        properties.ry = rect.r + 'pt';
    }
    return properties;
}
export function getCanvasEllipseProperties(ellipse, index) {
    var _a;
    const properties = Object.assign(Object.assign(Object.assign({}, getCanvasFilledElementProperties(ellipse, index)), getCanvasLineElementProperties(ellipse)), { cx: ellipse.x + 'pt', cy: ellipse.y + 'pt', rx: ellipse.r1 + 'pt', ry: ((_a = ellipse.r2) !== null && _a !== void 0 ? _a : ellipse.r1) + 'pt' });
    return properties;
}
export function getCanvasLineProperties(line) {
    const properties = Object.assign(Object.assign({}, getCanvasLineElementProperties(line)), { x1: line.x1 + 'pt', x2: line.x2 + 'pt', y1: line.y1 + 'pt', y2: line.y2 + 'pt' });
    if (line.lineCap) {
        properties['stroke-linecap'] = line.lineCap;
    }
    return properties;
}
export function getCanvasPolylineProperties(line, index) {
    if (!Array.isArray(line.points))
        return {};
    const points = line.closePath ? [...line.points, line.points[0]] : line.points;
    const properties = Object.assign(Object.assign(Object.assign({}, getCanvasFilledElementProperties(line, index)), getCanvasLineElementProperties(line)), { points: points
            .filter(point => typeof point === 'object' && !!point)
            .map(({ x, y }) => `${(x * 4) / 3},${(y * 4) / 3}`)
            .join(' ') });
    if (line.lineCap) {
        properties['stroke-linecap'] = line.lineCap;
    }
    return properties;
}
