import { getPageSize } from './page';
import { getStyleDictionary } from './style';
import { getStyleString } from './utils';
function getContentStyleDictionary(node, document, inline) {
    var _a, _b, _c, _d, _e;
    const pageSize = getPageSize(document);
    const maxWidth = pageSize.orientation === 'landscape' ? pageSize.height : pageSize.width;
    const styles = {
        'max-width': maxWidth + 'pt',
    };
    // some elements need display: block
    // - columns to grow properly
    // - nested lists for proper layouting
    // - the table of contents
    // - stacks and non-inline arrays
    if ((Array.isArray(node) && !inline) ||
        (typeof node === 'object' &&
            node &&
            ('columns' in node ||
                'ul' in node ||
                'ol' in node ||
                'toc' in node ||
                'stack' in node)))
        styles.display = 'block';
    else if (inline)
        styles.display = 'inline';
    // required for fontSize as otherwise the line-height would be wrong.
    // We cannot always assign inline-block because it messes up the vertical align of list markers
    else if (typeof node === 'object' &&
        node &&
        ('fontSize' in node || 'lineHeight' in node))
        styles.display = 'inline-block';
    // some elmeents do not apply backgrounds, only their children do
    if (Array.isArray(node) ||
        (typeof node === 'object' &&
            (node.stack || node.ul || node.ol || node.columns || node.qr))) {
        styles.background = 'transparent';
    }
    if (typeof node !== 'object' || !node || Array.isArray(node)) {
        return styles;
    }
    const applyBaseStyle = (nameOrStyle) => {
        var _a;
        if (typeof nameOrStyle === 'object') {
            Object.assign(styles, nameOrStyle);
            return;
        }
        const baseStyle = (_a = document.styles) === null || _a === void 0 ? void 0 : _a[nameOrStyle];
        if (baseStyle) {
            Object.assign(styles, getStyleDictionary(baseStyle));
        }
    };
    if (node.style) {
        if (typeof node.style === 'string') {
            applyBaseStyle(node.style);
        }
        else if (Array.isArray(node.style)) {
            node.style.forEach(applyBaseStyle);
        }
        else {
            Object.assign(styles, getStyleDictionary(node.style));
        }
    }
    Object.assign(styles, getStyleDictionary(node, true));
    if (inline) {
        Object.assign(styles, {
            display: 'inline',
            padding: '0',
        });
    }
    else {
        if (node.absolutePosition && typeof node.absolutePosition === 'object') {
            Object.assign(styles, {
                position: 'absolute',
                left: `${(_a = node.absolutePosition.x) !== null && _a !== void 0 ? _a : 0}pt`,
                top: `${(_b = node.absolutePosition.y) !== null && _b !== void 0 ? _b : 0}pt`,
            });
        }
        if (node.relativePosition && typeof node.relativePosition === 'object') {
            // relative positioning removes the space taken.
            // it can be combined with absolute positioning
            Object.assign(styles, {
                position: 'absolute',
                transform: `translate(${(_c = node.relativePosition.x) !== null && _c !== void 0 ? _c : 0}pt, ${(_d = node.relativePosition.y) !== null && _d !== void 0 ? _d : 0}pt)`,
            });
        }
        // for negative margin, we cannot use padding
        for (const suffix of ['', '-left', '-top', '-right', '-bottom']) {
            if ((_e = styles[`padding${suffix}`]) === null || _e === void 0 ? void 0 : _e.includes('-')) {
                styles[`margin${suffix}`] = styles[`padding${suffix}`];
                delete styles[`padding${suffix}`];
            }
        }
    }
    return styles;
}
export function getContentStyleString(node, document, inline = false) {
    const styles = getContentStyleDictionary(typeof node === 'string' || typeof node === 'number'
        ? { text: node }
        : node, document, inline);
    return getStyleString(styles);
}
