import { getStyleString } from './utils';
export function getUnorderedListStyleString(node) {
    var _a;
    const style = {
        'list-style-type': (_a = node.type) !== null && _a !== void 0 ? _a : 'disc',
    };
    if (node.markerColor)
        style['--marker-color'] = node.markerColor;
    return getStyleString(style);
}
export function getUnorderedListEntryStyleString(entry) {
    if (!entry.listType)
        return undefined;
    return getStyleString({
        'list-style-type': entry.listType,
    });
}
const romanChars = {
    1: 'I',
    5: 'V',
    10: 'X',
    50: 'L',
    100: 'C',
    500: 'D',
    1000: 'M',
};
function getRomanChars(num, start = 1000) {
    var _a, _b, _c, _d, _e, _f;
    let chars = '';
    let current = num;
    while (current >= start) {
        chars += (_a = romanChars[start]) !== null && _a !== void 0 ? _a : '';
        current -= start;
    }
    if (current < 1)
        return chars;
    if (current >= start * 0.9) {
        chars += (_b = romanChars[start / 10]) !== null && _b !== void 0 ? _b : '';
        chars += (_c = romanChars[start]) !== null && _c !== void 0 ? _c : '';
        current -= start * 0.9;
    }
    if (current < 1)
        return chars;
    if (current >= start / 2) {
        chars += (_d = romanChars[start / 2]) !== null && _d !== void 0 ? _d : '';
        current -= start / 2;
    }
    if (current >= start * 0.4) {
        chars += (_e = romanChars[start / 10]) !== null && _e !== void 0 ? _e : '';
        chars += (_f = romanChars[start / 2]) !== null && _f !== void 0 ? _f : '';
        current -= start * 0.4;
    }
    if (current < 1 || start < 10)
        return chars;
    return chars + getRomanChars(current, start / 10);
}
function getMaxCharacters(node) {
    var _a;
    if (!Array.isArray(node.ol))
        return 1;
    let largestNumber = ((_a = node.start) !== null && _a !== void 0 ? _a : 1) + node.ol.length;
    const counters = node.ol.filter(it => it.counter).map(it => it.counter);
    if (counters.length) {
        largestNumber = Math.max(largestNumber, ...counters);
    }
    switch (node.type) {
        case 'upper-alpha':
        case 'lower-alpha':
            return Math.ceil(Math.log(largestNumber) / Math.log(26));
        case 'upper-roman':
        case 'lower-roman':
            return getRomanChars(largestNumber).length;
        case 'none':
            return 1;
        case 'decimal':
        default:
            return Math.ceil(Math.log10(largestNumber));
    }
}
export function getOrderedListStyleString(node) {
    var _a, _b, _c, _d;
    // pdfmake applies padding according to the largest number;
    // HTML uses a fixed padding by default
    let characters = getMaxCharacters(node);
    const separator1 = node.separator && Array.isArray(node.separator) ? node.separator[0] : '';
    let separator2 = '.';
    if (node.separator) {
        separator2 = Array.isArray(node.separator)
            ? ((_a = node.separator[1]) !== null && _a !== void 0 ? _a : '')
            : node.separator;
    }
    characters += separator1.length + separator2.length;
    const style = {
        'padding-left': 0.25 + characters * 0.6 + 'em',
        '--separator1': JSON.stringify(separator1),
        '--separator2': JSON.stringify(separator2 + ' '),
    };
    const listType = (_b = node.type) !== null && _b !== void 0 ? _b : 'decimal';
    style['list-style-type'] = listType;
    style['--list-type'] = listType;
    if (listType === 'none') {
        style['--separator1'] = "''";
        style['--separator2'] = "''";
    }
    if (node.markerColor)
        style['--marker-color'] = node.markerColor;
    const startPosition = node.reversed
        ? ((_c = node.start) !== null && _c !== void 0 ? _c : node.ol.length) + 1
        : ((_d = node.start) !== null && _d !== void 0 ? _d : 1) - 1;
    if (startPosition !== 0)
        style['counter-reset'] = `item ${startPosition}`;
    // TODO pdfmake left-aligns the list markers
    return getStyleString(style);
}
export function getOrderedListEntryStyleString(entry) {
    if (entry.counter === undefined && !entry.listType)
        return undefined;
    const style = {};
    if (entry.counter !== undefined) {
        style['counter-set'] = 'custom ' + entry.counter;
    }
    if (entry.listType) {
        style['list-style-type'] = entry.listType;
        style['--list-type'] = entry.listType;
        if (entry.listType === 'none') {
            style['--separator1'] = "''";
            style['--separator2'] = "''";
        }
    }
    return getStyleString(style);
}
